package com.zh.project0512.controller.manage;

import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.model.dto.AdminRoleInsertDTO;
import com.zh.project0512.model.dto.AdminRoleListDTO;
import com.zh.project0512.model.dto.AdminRoleUpdateDTO;
import com.zh.project0512.model.vo.AdminRoleListVo;
import com.zh.project0512.service.IAdminRoleService;
import com.zh.project0512.utils.BizException;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import com.zh.project0512.utils.result.ResultPageInfo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * 管理账号角色
 */
@Tag(name = "管理账号角色")
@RestController
@RequestMapping("/manage/userRole")
public class AdminRoleController {
    @Resource
    IAdminRoleService userRoleService;


    /**
     * 查询用户角色
     * @param adminRoleListDTO 用户角色 list DTO
     * @return 用户角色
     */
    @Operation(summary = "查询角色")
    @PostMapping("/list")
    public ResultPageInfo<AdminRoleListVo> list(@Validated @RequestBody AdminRoleListDTO adminRoleListDTO){
        return ResultPageInfo.success(userRoleService.list(adminRoleListDTO));
    }

    /**
     * 添加用户角色
     * @param adminRoleInsertDTO 角色添加
     * @return 操作信息
     */
    @Operation(summary = "添加角色")
    @PostMapping("/insert")
    @AdminCheckAuthorityAnnotation(jurisdictionId = "128")
    public Result<String> insert(@Validated @RequestBody AdminRoleInsertDTO adminRoleInsertDTO){
        if (userRoleService.insert(adminRoleInsertDTO)){
            return Result.success("添加用户角色","操作成功");
        }else {
            throw new BizException(HttpStatusEnum.CUSTOM_EXCEPTION,"添加失败");
        }
    }

    /**
     * 修改用户角色
     * @param adminRoleUpdateDTO 角色 update DTO
     * @return 操作信息
     */
    @Operation(summary = "修改角色")
    @PostMapping("/update")
    @AdminCheckAuthorityAnnotation(jurisdictionId = "129")
    public Result<String> update(@Validated @RequestBody AdminRoleUpdateDTO adminRoleUpdateDTO){
        if (userRoleService.update(adminRoleUpdateDTO)){
            return Result.success("修改用户角色","操作成功");
        }else {
            throw new BizException(HttpStatusEnum.CUSTOM_EXCEPTION,"修改失败");
        }
    }

}
