package com.zh.project0512.controller.wxApp;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.zh.project0512.model.Reference;
import com.zh.project0512.model.ReferenceBrand;
import com.zh.project0512.model.ReferenceTag;
import com.zh.project0512.service.IReferenceBrandService;
import com.zh.project0512.service.IReferenceService;
import com.zh.project0512.service.IReferenceTagService;
import com.zh.project0512.utils.FileTypeUtil;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.jsonwebtoken.Claims;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.hibernate.validator.constraints.Range;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Digits;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-23
 */
@RestController
@RequestMapping("/wxApp/reference")
@Tag(name = "素材")
public class ReferenceUController {
    @Autowired
    private IReferenceService referenceService;
    @Autowired
    private IReferenceTagService referenceTagService;
    @Autowired
    private IReferenceBrandService referenceBrandService;

    @Data
    static class ListRParam {
        private int pageNum;
        private int pageSize;
        @Schema(title = "素材分组id")
        private Integer groupId;
        @Schema(title = "月份区间")
        private Integer month;
        @Schema(title = "开始时间(yyyy-MM-dd HH:mm:ss)",description = "如果传入month以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime beginTime;
        @Schema(title = "结束时间(yyyy-MM-dd HH:mm:ss)",description = "如果传入month以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime endTime;
        @Schema(title = "tagId数组")
        private List<Integer> tagIdList;
        @Schema(title = "brandId数组")
        private List<Integer> brandIdList;
        @Schema(title = "任务名称关键词搜索")
        private String keyword;
    }

    @Operation(summary = "素材列表")
    @PostMapping("/list")
    public Result list(@Validated @RequestBody ListRParam param, @RequestHeader(value = "token", required = false) @Parameter(name = "登录token") String token) {
        QueryWrapper<Reference> qw = new QueryWrapper<>();
        if (null != param.getGroupId()) {
            qw.eq("t1.groupId", param.getGroupId());
        }
        if (null != param.getMonth()) {
            qw.between("t1.creatAt", LocalDateTime.now().minusMonths(param.getMonth()),LocalDateTime.now());
        }else{
            qw.ge(param.getBeginTime()!=null,"t1.creatAt",param.getBeginTime());
            qw.le(param.getEndTime()!=null,"t1.creatAt",param.getEndTime());
        }
        if (null != param.getKeyword()) {
            qw.like("t1.title", param.getKeyword());
        }
        IPage iPage = referenceService.idList(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()),param.getTagIdList(),param.getBrandIdList(), qw);
        List list = iPage.getRecords();
        if (list.size() > 0) {
            iPage.setRecords(referenceService.list(new JwtUtil().parseOpenid(token),list,param.getTagIdList(),param.getBrandIdList(), qw));
        }
        return Result.success(iPage);
    }

    @Operation(summary = "素材列表按日分组")
    @PostMapping("/listByDay")
    public Result listByDay(@Validated @RequestBody ListRParam param, @RequestHeader(value = "token", required = false) @Parameter(name = "登录token") String token) {
        QueryWrapper<Reference> qw = new QueryWrapper<>();
        if (null != param.getGroupId()) {
            qw.eq("t1.groupId", param.getGroupId());
        }
        if (null != param.getMonth()) {
            qw.between("t1.creatAt", LocalDateTime.now().minusMonths(param.getMonth()),LocalDateTime.now());
        }else{
            qw.ge(param.getBeginTime()!=null,"t1.creatAt",param.getBeginTime());
            qw.le(param.getEndTime()!=null,"t1.creatAt",param.getEndTime());
        }
        if (null != param.getKeyword()) {
            qw.like("t1.title", param.getKeyword());
        }
        IPage iPage = referenceService.dateList(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()),param.getTagIdList(),param.getBrandIdList(), qw);
        List list = iPage.getRecords();
        if (list.size() > 0) {
            iPage.setRecords(referenceService.referenceDateList(new JwtUtil().parseOpenid(token),list,param.getTagIdList(),param.getBrandIdList(), qw));
        }
        return Result.success(iPage);
    }

    @Data
    static class DetPParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "素材id")
        private Integer id;
    }

    @Operation(summary = "详情")
    @PostMapping("/detail")
    public Result detail(@Validated @RequestBody DetPParam param) {
        int id = param.getId();
        Reference reference = referenceService.getById(id);
        return reference != null ? Result.success(reference.setBrandList(referenceBrandService.selByReferenceId(id)).setTagList(referenceTagService.selByReferenceId(id))) : Result.fail(HttpStatusEnum.NOT_FOUND);
    }

}
