package com.zh.project0512.controller.wxApp;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.annotation.TokenValid;
import com.zh.project0512.model.User;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.utils.*;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-18
 */
@Tag(name = "用户")
@RestController
@RequestMapping("/wxApp/user")
public class UserUController {

    @Autowired
    private IUserService userService;
    @Autowired
    private WeChatUtil weChatUtil;
    @Autowired
    private HttpUtil httpUtil;
    @Operation(summary = "小程序登录", parameters = {@Parameter(name = "code", description = "login返回的code（未使用过！）")})
    @PostMapping("/login")
    public Result login(@RequestBody @Parameter(hidden = true) JSONObject obj) {
        JSONObject res = weChatUtil.auth(obj.getString("code"));
        String openid = res.getString("openid");
        Map u = userService.selByOpenid2(openid);
        if (null == u) {
            return Result.fail(HttpStatusEnum.USER_NOT_EXIST, "未注册！");
        }
        String token = new JwtUtil().createJWT(openid);
        userService.updByOpenid(openid, token, LocalDateTime.now());
        u.replace("token",token);
        return Result.success(u);
    }

    @Operation(summary = "小程序注册", parameters = {
        @Parameter(name = "code", description = "login返回的code（未使用过！）"),
        @Parameter(name = "userInfo", description = "getUserInfo接口的iv和encryptedData"),
        @Parameter(name = "phone", description = "getPhoneNumber接口的iv和encryptedData")
    })
    @PostMapping("register")
    public Result register(@RequestBody @Parameter(hidden = true) JSONObject obj) {
        JSONObject res = weChatUtil.auth(obj.getString("code"));
        String openid = res.getString("openid");
        if (null == openid) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "无效的openid");
        }
        if (null != userService.selByOpenid(openid)) {
            return Result.fail(HttpStatusEnum.TEL_REGISTERED, "已注册");
        }
        String session_key = (String) res.get("session_key");
        JSONObject userinfo = obj.getJSONObject("userinfo");
        JSONObject phone = obj.getJSONObject("phone");
        String info = CoderUtil.decrypt(session_key, userinfo.getString("iv"), userinfo.getString("encryptedData"));
        String info2 = CoderUtil.decrypt(session_key, phone.getString("iv"), phone.getString("encryptedData"));
        JSONObject j = JSONObject.parseObject(info);
        JSONObject k = JSONObject.parseObject(info2);
        if (null == j || null == k) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "session_key解析失败");
        }
        User u = new User();
        u.setAvatar((String) j.get("avatarUrl"));
        u.setName((String) j.get("nickName"));
        u.setGender((Integer) j.get("gender"));
        u.setOpenid(openid);
        u.setTel(k.getString("phoneNumber"));
        u.setToken(new JwtUtil().createJWT(openid));
        u.setCreatAt(LocalDateTime.now());

        // 用户企业微信信息获取
//        String access_token = httpUtil.qywxGetToken();
//        JSONObject qywxRes = httpUtil.qywxUserSession(access_token, obj.getString("code"));
//        String userid = qywxRes.getString("userid");
//        JSONObject res1 = httpUtil.qywxUser(access_token, userid);
//        Integer main_department = res1.getInteger("main_department");
//        JSONArray department = res1.getJSONArray("department");
//
//        if (null == userid || null == main_department) {
//            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "获取企业用户信息失败");
//        }
//        Integer sub_department = main_department;
//        if (sub_department != null && sub_department != 1 && sub_department != 0) {
//            sub_department = httpUtil.qywxDepartmentParentid(access_token, main_department, 0);
//        }
//        u.setMain_department(main_department).setSub_department(sub_department);
        userService.save(u);
        return Result.success(u);
    }
    @PostMapping("/logout")
    @TokenValid
    public Result logout(@RequestHeader("token") @Parameter(name = "登录token") String token) {
        UpdateWrapper<User> uw = new UpdateWrapper<>();
        uw.eq("token",token).set("token",null).set("updateAt",LocalDateTime.now());
        return MybatisPlusUtil.sqlResult(userService.update(uw),"登出");
    }
}
